from __future__ import annotations

from typing import List, Optional

from sqlalchemy.orm import Session

from app.models.content import Subject
from app.schemas.content import SubjectCreate, SubjectUpdate
from app.utils.db import CRUDBase


class CRUDSubject(CRUDBase[Subject, SubjectCreate, SubjectUpdate]):
    def get_by_name(self, db: Session, *, name: str) -> Optional[Subject]:
        """
        Get a subject by name.
        """
        return db.query(Subject).filter(Subject.name == name).first()

    def get_active(self, db: Session, *, skip: int = 0, limit: int = 100) -> List[Subject]:
        """
        Get active subjects.
        """
        return (
            db.query(Subject)
            .filter(Subject.is_active == True)
            .order_by(Subject.order)
            .offset(skip)
            .limit(limit)
            .all()
        )


subject = CRUDSubject(Subject)
