"""Initial schema

Revision ID: 001
Revises: 
Create Date: 2023-08-01 00:00:00.000000

"""
from alembic import op
import sqlalchemy as sa
from sqlalchemy.dialects import sqlite

# revision identifiers, used by Alembic.
revision = '001'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    # Create user table
    op.create_table(
        'user',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('email', sa.String(), nullable=False),
        sa.Column('username', sa.String(), nullable=False),
        sa.Column('full_name', sa.String(), nullable=True),
        sa.Column('hashed_password', sa.String(), nullable=False),
        sa.Column('is_active', sa.Boolean(), nullable=True, default=True),
        sa.Column('is_superuser', sa.Boolean(), nullable=True, default=False),
        sa.Column('date_of_birth', sa.DateTime(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.Column('points', sa.Integer(), nullable=True, default=0),
        sa.Column('level', sa.Integer(), nullable=True, default=1),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_user_email'), 'user', ['email'], unique=True)
    op.create_index(op.f('ix_user_id'), 'user', ['id'], unique=False)
    op.create_index(op.f('ix_user_username'), 'user', ['username'], unique=True)

    # Create subject table
    op.create_table(
        'subject',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('name', sa.String(), nullable=False),
        sa.Column('description', sa.Text(), nullable=True),
        sa.Column('image_url', sa.String(), nullable=True),
        sa.Column('order', sa.Integer(), nullable=True, default=0),
        sa.Column('is_active', sa.Boolean(), nullable=True, default=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_subject_id'), 'subject', ['id'], unique=False)
    op.create_index(op.f('ix_subject_name'), 'subject', ['name'], unique=False)

    # Create lesson table
    op.create_table(
        'lesson',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('subject_id', sa.Integer(), nullable=False),
        sa.Column('title', sa.String(), nullable=False),
        sa.Column('description', sa.Text(), nullable=True),
        sa.Column('content', sa.Text(), nullable=False),
        sa.Column('image_url', sa.String(), nullable=True),
        sa.Column('order', sa.Integer(), nullable=True, default=0),
        sa.Column('difficulty', sa.Enum('easy', 'medium', 'hard', name='difficultylevel'), nullable=True),
        sa.Column('points', sa.Integer(), nullable=True, default=10),
        sa.Column('duration_minutes', sa.Integer(), nullable=True, default=15),
        sa.Column('is_active', sa.Boolean(), nullable=True, default=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.ForeignKeyConstraint(['subject_id'], ['subject.id'], ),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_lesson_id'), 'lesson', ['id'], unique=False)
    op.create_index(op.f('ix_lesson_title'), 'lesson', ['title'], unique=False)

    # Create quiz table
    op.create_table(
        'quiz',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('lesson_id', sa.Integer(), nullable=False),
        sa.Column('title', sa.String(), nullable=False),
        sa.Column('description', sa.Text(), nullable=True),
        sa.Column('time_limit_minutes', sa.Integer(), nullable=True, default=10),
        sa.Column('pass_percentage', sa.Integer(), nullable=True, default=70),
        sa.Column('points', sa.Integer(), nullable=True, default=20),
        sa.Column('is_active', sa.Boolean(), nullable=True, default=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.ForeignKeyConstraint(['lesson_id'], ['lesson.id'], ),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_quiz_id'), 'quiz', ['id'], unique=False)
    op.create_index(op.f('ix_quiz_title'), 'quiz', ['title'], unique=False)

    # Create question table
    op.create_table(
        'question',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('quiz_id', sa.Integer(), nullable=False),
        sa.Column('text', sa.Text(), nullable=False),
        sa.Column('image_url', sa.String(), nullable=True),
        sa.Column('explanation', sa.Text(), nullable=True),
        sa.Column('points', sa.Integer(), nullable=True, default=5),
        sa.Column('order', sa.Integer(), nullable=True, default=0),
        sa.Column('is_active', sa.Boolean(), nullable=True, default=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.ForeignKeyConstraint(['quiz_id'], ['quiz.id'], ),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_question_id'), 'question', ['id'], unique=False)

    # Create answer table
    op.create_table(
        'answer',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('question_id', sa.Integer(), nullable=False),
        sa.Column('text', sa.Text(), nullable=False),
        sa.Column('is_correct', sa.Boolean(), nullable=True, default=False),
        sa.Column('explanation', sa.Text(), nullable=True),
        sa.Column('order', sa.Integer(), nullable=True, default=0),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.ForeignKeyConstraint(['question_id'], ['question.id'], ),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_answer_id'), 'answer', ['id'], unique=False)

    # Create achievement table
    op.create_table(
        'achievement',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('name', sa.String(), nullable=False),
        sa.Column('description', sa.Text(), nullable=False),
        sa.Column('type', sa.Enum('completion', 'streak', 'performance', 'milestone', 'special', name='achievementtype'), nullable=False),
        sa.Column('image_url', sa.String(), nullable=True),
        sa.Column('points', sa.Integer(), nullable=True, default=0),
        sa.Column('criteria', sa.Text(), nullable=False),
        sa.Column('is_active', sa.Boolean(), nullable=True, default=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_index(op.f('ix_achievement_id'), 'achievement', ['id'], unique=False)
    op.create_index(op.f('ix_achievement_name'), 'achievement', ['name'], unique=False)

    # Create user_progress table
    op.create_table(
        'userprogress',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('user_id', sa.Integer(), nullable=False),
        sa.Column('lesson_id', sa.Integer(), nullable=False),
        sa.Column('status', sa.Enum('not_started', 'in_progress', 'completed', name='progressstatus'), nullable=True),
        sa.Column('progress_percentage', sa.Float(), nullable=True, default=0.0),
        sa.Column('completed_at', sa.DateTime(), nullable=True),
        sa.Column('points_earned', sa.Integer(), nullable=True, default=0),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.ForeignKeyConstraint(['lesson_id'], ['lesson.id'], ),
        sa.ForeignKeyConstraint(['user_id'], ['user.id'], ),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('user_id', 'lesson_id', name='user_lesson_progress_uc')
    )
    op.create_index(op.f('ix_userprogress_id'), 'userprogress', ['id'], unique=False)

    # Create user_achievement table
    op.create_table(
        'userachievement',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('user_id', sa.Integer(), nullable=False),
        sa.Column('achievement_id', sa.Integer(), nullable=False),
        sa.Column('earned_at', sa.DateTime(), nullable=True),
        sa.ForeignKeyConstraint(['achievement_id'], ['achievement.id'], ),
        sa.ForeignKeyConstraint(['user_id'], ['user.id'], ),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('user_id', 'achievement_id', name='user_achievement_uc')
    )
    op.create_index(op.f('ix_userachievement_id'), 'userachievement', ['id'], unique=False)

    # Create user_answer table
    op.create_table(
        'useranswer',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('user_id', sa.Integer(), nullable=False),
        sa.Column('question_id', sa.Integer(), nullable=False),
        sa.Column('answer_id', sa.Integer(), nullable=False),
        sa.Column('is_correct', sa.Boolean(), nullable=True, default=False),
        sa.Column('points_earned', sa.Integer(), nullable=True, default=0),
        sa.Column('attempt_number', sa.Integer(), nullable=True, default=1),
        sa.Column('time_taken_seconds', sa.Integer(), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.ForeignKeyConstraint(['answer_id'], ['answer.id'], ),
        sa.ForeignKeyConstraint(['question_id'], ['question.id'], ),
        sa.ForeignKeyConstraint(['user_id'], ['user.id'], ),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('user_id', 'question_id', 'attempt_number', name='user_question_attempt_uc')
    )
    op.create_index(op.f('ix_useranswer_id'), 'useranswer', ['id'], unique=False)


def downgrade():
    # Drop all tables in reverse order
    op.drop_index(op.f('ix_useranswer_id'), table_name='useranswer')
    op.drop_table('useranswer')
    
    op.drop_index(op.f('ix_userachievement_id'), table_name='userachievement')
    op.drop_table('userachievement')
    
    op.drop_index(op.f('ix_userprogress_id'), table_name='userprogress')
    op.drop_table('userprogress')
    
    op.drop_index(op.f('ix_achievement_name'), table_name='achievement')
    op.drop_index(op.f('ix_achievement_id'), table_name='achievement')
    op.drop_table('achievement')
    
    op.drop_index(op.f('ix_answer_id'), table_name='answer')
    op.drop_table('answer')
    
    op.drop_index(op.f('ix_question_id'), table_name='question')
    op.drop_table('question')
    
    op.drop_index(op.f('ix_quiz_title'), table_name='quiz')
    op.drop_index(op.f('ix_quiz_id'), table_name='quiz')
    op.drop_table('quiz')
    
    op.drop_index(op.f('ix_lesson_title'), table_name='lesson')
    op.drop_index(op.f('ix_lesson_id'), table_name='lesson')
    op.drop_table('lesson')
    
    op.drop_index(op.f('ix_subject_name'), table_name='subject')
    op.drop_index(op.f('ix_subject_id'), table_name='subject')
    op.drop_table('subject')
    
    op.drop_index(op.f('ix_user_username'), table_name='user')
    op.drop_index(op.f('ix_user_id'), table_name='user')
    op.drop_index(op.f('ix_user_email'), table_name='user')
    op.drop_table('user')
    
    # Drop the enum types
    sa.Enum(name='difficultylevel').drop(op.get_bind(), checkfirst=False)
    sa.Enum(name='achievementtype').drop(op.get_bind(), checkfirst=False)
    sa.Enum(name='progressstatus').drop(op.get_bind(), checkfirst=False)